﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Media.Imaging;
using System.Windows.Forms;
using System.Linq;
using System.IO;
using WpfAnimatedGif;

namespace RandomImage
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		string _seaerchDirectoryPath;
		bool _isFirstSearch;
		List<string> _randomizedImagesPaths;
		int _currentCollectionIndex;
		int _maximalCollectionIndex;
		string _selectedImagePath;

		public MainWindow()
		{
			InitializeComponent();
			_isFirstSearch = true;
			_seaerchDirectoryPath = Environment.CurrentDirectory;
			_randomizedImagesPaths = new List<string>();
			_currentCollectionIndex = 0;
			_maximalCollectionIndex = 0;
		}

		private void Button_Click(object sender, RoutedEventArgs e)
		{
			var dialog = new FolderBrowserDialog();
			DialogResult result = dialog.ShowDialog();
			if (result.ToString().ToLower().Equals("ok"))
			{
				_seaerchDirectoryPath = dialog.SelectedPath;
				Selected_path_label.Content = _seaerchDirectoryPath;
				_isFirstSearch = true;
			}
		}

		private void Next_btn_Click(object sender, RoutedEventArgs e)
		{
			if (_isFirstSearch)
				FirstSearch();

			if (_maximalCollectionIndex == -1)
			{
				System.Windows.MessageBox.Show("Sorry, but I found 0 images, try another directory. :3");
				_isFirstSearch = true;
				return;
			}

			if (_currentCollectionIndex == _maximalCollectionIndex)
				_currentCollectionIndex = 0;
			else
				_currentCollectionIndex++;

			DisplayImage(_randomizedImagesPaths.ElementAt(_currentCollectionIndex));
			_selectedImagePath = _randomizedImagesPaths.ElementAt(_currentCollectionIndex);
		}

		private void Prev_btn_Click(object sender, RoutedEventArgs e)
		{
			if (_isFirstSearch)
				FirstSearch();

			if (_maximalCollectionIndex == -1)
			{
				System.Windows.MessageBox.Show("Sorry, but I found 0 images, try another directory. :3");
				_isFirstSearch = true;
				return;
			}

			if (_currentCollectionIndex == 0)
				_currentCollectionIndex = _maximalCollectionIndex;
			else
				_currentCollectionIndex--;

			DisplayImage(_randomizedImagesPaths.ElementAt(_currentCollectionIndex));
			_selectedImagePath = _randomizedImagesPaths.ElementAt(_currentCollectionIndex);
		}

		public static void Shuffle<T>(List<T> list)
		{
			Random rng = new Random(DateTime.Now.Millisecond / ((list.Count * 42) + 2));
			int n = list.Count;
			while (n > 1)
			{
				n--;
				int k = rng.Next(n + 1);
				T value = list[k];
				list[k] = list[n];
				list[n] = value;
			}
		}

		private void FirstSearch()
		{
			List<string> searchPatterns = new List<string>();

			//searchPatterns.Add("*.jpeg");
			//searchPatterns.Add("*.png");
			//searchPatterns.Add("*.gif");
			searchPatterns.Add("*");
			if (!Directory.Exists(_seaerchDirectoryPath))
			{
				System.Windows.MessageBox.Show("Selected directory doesn't exist anymore, choose another. :3");
				return;
			}
			_randomizedImagesPaths.Clear();

			//SearchOption opt = Subdirs_ck.IsChecked == true ? SearchOption.AllDirectories : SearchOption.TopDirectoryOnly;
			//try
			//{
			//	var sch1 = Directory.GetFiles(_seaerchDirectoryPath, "*.jp*g", opt);
			//	var sch2 = Directory.GetFiles(_seaerchDirectoryPath, "*.png", opt);
			//	var sch3 = Directory.GetFiles(_seaerchDirectoryPath, "*.gif", opt);
			//	_randomizedImagesPaths = sch1.Concat(sch2).Concat(sch3).ToList();
			//}
			//catch (Exception)
			//{
			//	System.Windows.MessageBox.Show("Seems like we have unauthorized access here, choose another directory. :3");
			//}
			bool opt = !Subdirs_ck.IsChecked.Value;
			string oldCap = WindowArea.Title;
			WindowArea.Title += " ( In Progress... )";
			Search(_seaerchDirectoryPath, ref searchPatterns, ref _randomizedImagesPaths, ref opt);
			Shuffle<string>(_randomizedImagesPaths);
			Shuffle<string>(_randomizedImagesPaths);
			Shuffle<string>(_randomizedImagesPaths);
			_isFirstSearch = false;
			_maximalCollectionIndex = _randomizedImagesPaths.Count - 1;
			Found_lbl.Content = _randomizedImagesPaths.Count;
			WindowArea.Title = oldCap;
		}

		private void Search(string dir, ref List<string> searchPatterns, ref List<string> foundFiles, ref bool topDirOnly)
		{
			String[] subdirs = { };
			DirectoryInfo di = new DirectoryInfo(dir);
			List<FileInfo> fiArray = new List<FileInfo>();

			//if (!Directory.Exists(dir))
			//return;

			if (
			dir.StartsWith("C:\\Windows") ||
			dir.StartsWith("C:\\Program Files") ||
			dir.StartsWith("C:\\Program Files (x86)") ||
			dir.StartsWith("C:\\ProgramData") ||
			dir.StartsWith("C:\\$Recycle.Bin") ||
			dir.StartsWith("C:\\Boot"))
				return;

			if (!topDirOnly)
			{
				try
				{
					subdirs = Directory.GetDirectories(dir, "*", SearchOption.TopDirectoryOnly);
				}
				catch (Exception e) { return; }
			}

			foreach (string subdir in subdirs)
				Search(subdir, ref searchPatterns, ref foundFiles, ref topDirOnly);

			foreach (string pattern in searchPatterns)
				fiArray.AddRange(di.GetFiles(pattern, SearchOption.TopDirectoryOnly));
			foreach (FileInfo fi in fiArray)
			{
				try
				{
					string path = fi.FullName;
					if (path.EndsWith(".jpg") || path.EndsWith(".png") || path.EndsWith(".gif") || path.EndsWith(".jpeg"))
						//foundFiles.Add(fi.FullName);
						foundFiles.Add(path);
				}
				catch (Exception e) { }
			}
		}

		private void Add_btn_Click(object sender, RoutedEventArgs e)
		{
			if (Selected_images_txt.Text.Length != 0)
				Selected_images_txt.Text += Environment.NewLine;
			if (Selected_images_txt.Text.Contains(_selectedImagePath))
			{
				System.Windows.MessageBox.Show("Seems like this pictures path is already added. :3");
				return;
			}
			Selected_images_txt.Text += _selectedImagePath;
		}

		private void Subdirs_ck_Checked(object sender, RoutedEventArgs e)
		{
			FirstSearch();
		}

		private void Subdirs_ck_Unchecked(object sender, RoutedEventArgs e)
		{
			FirstSearch();
		}

		private void DisplayImage(string imagePath)
		{
			//Preview_img.Source = new BitmapImage(new Uri(imagePath));
			try
			{
				var image = new BitmapImage();
				image.BeginInit();
				image.UriSource = new Uri(imagePath);
				image.EndInit();
				ImageBehavior.SetAnimatedSource(Preview_img, image);
			}
			catch (Exception e)
			{
				System.Windows.MessageBox.Show(e.Message, imagePath);
			}
		}

	}
}
